
#include "Ble Test1 Defs.h"
#include "services.h"
#include "acilib_prototypes.h"
#include "acilib.h"

//sbit BlueLE_BUTTON at GPIOB_ODR.B0;
sbit BlueLE_RST at GPIOC_ODR.B2;
sbit BlueLE_BONDED at GPIOD_ODR.B14;
sbit BlueLE_RDY at GPIOD_IDR.B10;       //PD10
sbit BlueLE_REQ at GPIOD_ODR.B13;
sbit BlueLE_ACT at GPIOA_IDR.B4;
sbit BlueLE_CONN at GPIOA_ODR.B0;

short readBuff[32] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
char writeBuff[32];
char eventBuff[32];

//short open_adv_pipe[10] = {0x09,0x1B,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
short bond[6] = {0x05,0x10,0xB4,0x00,0x20,0x00};
short connect[6] = {0x05,0x0F,0x00,0x00,0x20,0x00};
short Directconnect[2] = {0x01,0x1E};
short broadcast[6] = {0x05,0x1C,0x00,0x00,0x00,0x20};
short set_local_data[4] = {0x03,0x0D,0x02,0xAA};
char  sendData[] = {0x4, 0x15, 1, 0xaa, 0xbb};

volatile aci_evt_t p_aci_evt;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void InitGPIOInt();
void Init_MCU();
void BlueLE_Write(unsigned short *dat, unsigned int len);
void BlueLE_Read();
void BlueLE_Setup();
char BlueLE_WaitForEvent(char event);

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void InitGPIOInt() {
  GPIO_Digital_Input(&GPIOA_BASE, _GPIO_PINMASK_4);

//  RCC_APB2ENR.AFIOEN = 1;              // Enable clock for alternate pin functions
  AFIO_EXTICR1 = 0x0000;               // PD10 as External interrupt
  EXTI_FTSR = 0x10;              // Set interrupt on Falling edge
  EXTI_IMR |= 0x10;              // Set mask
//  NVIC_IntEnable(IVT_INT_EXTI4);
  //EnableInterrupts();                  // Enables the processor interrupt.
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void Init_MCU() {
   //GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_ALL);                 // Led port

   GPIO_Digital_Output(&GPIOC_ODR, _GPIO_PINMASK_2);                                // Set PC2 (RST) as digital output
   GPIO_Digital_Input(&GPIOA_IDR, _GPIO_PINMASK_4);                               // Set PA4 (ACT) as digital input
   GPIO_Digital_Output(&GPIOD_ODR, _GPIO_PINMASK_13 | _GPIO_PINMASK_14);                             // Set PD13 (REQ) as digital output
   GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_10);                              // Set PD10 (RDY) as digital input
   GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_0);            // Set PA0 (CONN)
   
   UART1_Init(56000);

   SPI3_Init_Advanced(_SPI_FPCLK_DIV16, _SPI_MASTER | _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION | _SPI_LSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12);
   BlueLE_REQ = 1;
   BlueLE_RST = 1;
   Delay_ms(100);
   BlueLE_RST = 0;
   Delay_ms(100);
   BlueLE_RST = 1;
   
   //UART_Write_Line("Start");
   InitGPIOInt();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void BlueLE_Write(unsigned short *dat, unsigned int txLen)  {
char tmp, cnt, rxLen, rcvDummy, numOfRead;

  BlueLE_REQ = 0;
  while(BlueLE_RDY == 1);

  rcvDummy = SPI_Read(dat[0]);
  rxLen = SPI_Read(dat[1]);
  if(rxLen != 0) {
    readBuff[0] = rxLen;
    //UART_Write(rxLen);
  }
  
  for (cnt=2; (cnt<txLen + 1) || (cnt<rxLen + 2); cnt++)
  {
  
    if (cnt < txLen + 1) {
        tmp = SPI_Read(dat[cnt]); // transmit
    } else {
        tmp = SPI_Read(0); // receive only
    }

    if (cnt < rxLen + 2) { // receive
//        UART_Write(tmp);
        readBuff[cnt-1] = tmp;
    }
//    Delay_ms(10);
  }

  BlueLE_REQ = 1;
//  while(BlueLE_RDY == 0);
  if(rxLen > 0) {
    acil_decode_evt(readBuff, &p_aci_evt);
    
    if(p_aci_evt.evt_opcode == ACI_EVT_DATA_RECEIVED)
    {
        asm nop;
        // Receive Data over UART_RX
        //UART_Write();
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void BlueLE_Read() {
char cnt, dummy;

  while(BlueLE_RDY);
  BlueLE_REQ = 0;

  dummy =  SPI_Read(dummy);
  readBuff[0] = SPI_Read(dummy);
  if(readBuff[0] > 0) {
    //UART_Write(readBuff[0]);
    for (cnt=1; cnt<=readBuff[0]; cnt++){
        readBuff[cnt] = SPI_Read(dummy);
//        Delay_ms(10);
        //UART_Write(readBuff[cnt]);
    }
    asm nop;
  }
  BlueLE_REQ = 1;
  acil_decode_evt(readBuff, &p_aci_evt);

  if(p_aci_evt.evt_opcode == ACI_EVT_DATA_RECEIVED)
  {
      asm nop;
      // Receive Data over UART_RX
      //p_aci_evt.params.data_received.rx_data
  }
}

void BlueLe_WaitRead() {
   while(BlueLE_RDY == 1);
   while(BlueLE_RDY == 0);
}

////////////////////////////////////////////////////////////////////////////////
//                              GET EVENT CODE
////////////////////////////////////////////////////////////////////////////////

char BlueLE_EventCode() {
  char cnt, dummy;

  while(BlueLE_RDY);
  BlueLE_REQ = 0;

  dummy =  SPI_Read(dummy);
  readBuff[0] = SPI_Read(dummy);
  if(readBuff[0] > 0) {
    //UART_Write(readBuff[0]);
    for (cnt=1; cnt<=readBuff[0]; cnt++){
        readBuff[cnt] = SPI_Read(dummy);
//        Delay_ms(10);
        //UART_Write(readBuff[cnt]);
    }
    asm nop;
  }
  BlueLE_REQ = 1;
  acil_decode_evt(readBuff, &p_aci_evt);
  return p_aci_evt.evt_opcode;
}



////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

char BlueLE_WaitForEvent(char event) {

  while(1) {
    BlueLE_Read();
    if(p_aci_evt.evt_opcode == event) {
      break;
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
aci_cmd_params_bond_t bondParam;
void BlueLE_Bond(uint16_t timeout, uint16_t adv_interval) {
  bondParam.timeout = timeout;
  bondParam.adv_interval = adv_interval;
  acil_encode_cmd_bond(writeBuff, (aci_cmd_params_bond_t*)&bondParam);
  BlueLE_Write(writeBuff, MSG_BOND_LEN);
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void BlueLe_DirectConnect() {
  acil_encode_direct_connect(writeBuff);
  BlueLE_Write(writeBuff, 2);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

aci_cmd_params_connect_t p_aci_cmd_params_connect;
void BlueLe_Connect() {
  p_aci_cmd_params_connect.timeout = 0;
  p_aci_cmd_params_connect.adv_interval = 0x20;
  acil_encode_cmd_connect(writeBuff, &p_aci_cmd_params_connect);
  BlueLE_Write(writeBuff, MSG_CONNECT_LEN);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

aci_cmd_params_disconnect_t p_aci_cmd_params_disconnect;
void BlueLe_Disconnect() {
  p_aci_cmd_params_disconnect.reason = 0;
  acil_encode_cmd_disconnect(writeBuff, &p_aci_cmd_params_disconnect);
  BlueLE_Write(writeBuff, MSG_CONNECT_LEN);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

aci_cmd_params_send_data_t p_aci_cmd_params_send_data_t;
void BlueLE_SendData(uint8_t *dat, uint8_t len, uint8_t pipe) {

  p_aci_cmd_params_send_data_t.tx_data.pipe_number = pipe;
  memcpy(p_aci_cmd_params_send_data_t.tx_data.aci_data, dat, len);
  acil_encode_cmd_send_data(writeBuff, &p_aci_cmd_params_send_data_t, len);
  BlueLE_Write(writeBuff, MSG_SEND_DATA_BASE_LEN + len);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

aci_cmd_params_open_remote_pipe_t p_aci_cmd_params_open_remote_pipe;
void BlueLE_OpenRemotePipe(uint8_t pipe) {
  p_aci_cmd_params_open_remote_pipe.pipe_number = pipe;
  acil_encode_cmd_open_remote_pipe(writeBuff, &p_aci_cmd_params_open_remote_pipe);

  BlueLE_Write(writeBuff, MSG_OPEN_REMOTE_PIPE_LEN);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void BlueLE_SendDataAck(uint8_t pipe) {
  acil_encode_cmd_send_data_ack(writeBuff, pipe );
  BlueLE_Write(writeBuff, MSG_ACK_LEN);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void BlueLE_GetTemp() {
  acil_encode_cmd_temparature(writeBuff);
  BlueLE_Write(writeBuff, 1);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static unsigned int tempValue;
char BlueLE_DecodeTemp() {

  BlueLE_GetTemp();
  BlueLE_WaitForEvent(ACI_EVT_CMD_RSP);
  tempValue = 0x8000;
  
  if(p_aci_evt.params.cmd_rsp.cmd_opcode == ACI_CMD_GET_TEMPERATURE) {
    if(p_aci_evt.params.cmd_rsp.cmd_status == 0) {
      tempValue = (p_aci_evt.params.cmd_rsp.params.get_temperature.temperature_value) >> 2;
    }
  }
  return (char)tempValue;
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

aci_cmd_params_set_local_data_t p_aci_cmd_params_set_local_data;
void BlueLE_SetLocalData(char *dat, char len, char pipe) {
  p_aci_cmd_params_set_local_data.tx_data.pipe_number = pipe;
  memcpy(p_aci_cmd_params_set_local_data.tx_data.aci_data, dat, len);
  acil_encode_cmd_set_local_data(writeBuff, &p_aci_cmd_params_set_local_data, len);
  BlueLE_Write(writeBuff, MSG_SET_LOCAL_DATA_BASE_LEN + len);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

aci_cmd_params_test_t p_aci_cmd_params_test;
void BlueLE_TestModeState(char state) {
  p_aci_cmd_params_test.test_mode_change = state;
  acil_encode_cmd_set_test_mode(writeBuff, &p_aci_cmd_params_test);
  BlueLE_Write(writeBuff, 2);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

const TSetupMsg setupMsg[NB_SETUP_MESSAGES] = SETUP_MESSAGES_CONTENT;
void BlueLE_Setup() {

char reqLen;
char numOfMsg = 0;

//   NVIC_IntDisable(IVT_INT_EXTI4);

   BlueLE_WaitForEvent(ACI_EVT_DEVICE_STARTED);
   
   Delay_ms(200);
   BlueLE_TestModeState(0xff);

   while(numOfMsg < NB_SETUP_MESSAGES)
   {
//     reqLen = setupMsg[numOfMsg].buffer[0] + 1;
     reqLen = setupMsg[numOfMsg].buffer[0];
     BlueLE_Write(setupMsg[numOfMsg].buffer, reqLen);
     BlueLE_Read();
     numOfMsg++;
   }

   BlueLE_WaitForEvent(ACI_EVT_DEVICE_STARTED);
//   NVIC_IntEnable(IVT_INT_EXTI4);
}

/*
void UART_Write_Line(char *line)
{
  UART_Write(13);
  UART_Write(10);
  UART_Write_Text(line);
}*/