
#include "stdint.h"

/*SYSTEM COMMANDS*/

#define _TEST 0x01              //   Enables test mode
#define _ECHO 0x02              //   Echo test
#define _DTM_COMM 0x03          //   Direct test mode command
#define _SLEEP 0x04             //   Activates SLEEP mode
#define _WAKEUP 0x05            //   Wakes up from sleep mode
#define _SETUP 0x06             //   Uploads the configuration generated by nRFgo Studio
                                //     -- GAP  Settings
                                //     -- GATT Services
                                //     -- HARDWARE Settings
#define _READ_DYN_DAT 0x07      //   Extracts dynamic data for storage in controller
#define _WRITE_DYN_DAT 0x08     //   Restores dynamic data to nRF8001 volatile memory
#define _DEV_VER 0x09           //   Device version
#define _DEV_ADD 0x0A           //   Device address
#define _BATT_LEV 0x0B          //   Measures the battery supply voltage
#define _TEMP 0x0C              //   Measures the ambient temperature
#define _RAD_RES 0x0E           //   Reset the radio transceiver
#define _CONNECT 0x0F           //   Start advertising and establishes a coonection with the peer device
#define _BOND 0x10              //   Start advertising with the intent os setting up a trusted relationship with a peer device
#define _DISCONNECT 0x11        //   Terminates the connection with the peer device
#define _TX_POW 0x12            //   Output power level of radio
#define _TIME_REQ 0x13          //   Initiates the connection parameter update procedure
#define _OPEN_REM_PIPE 0x14     //   Opens a remote receive pipe from a peer device for data transfer
#define _APP_LAT 0x19           //   Sets the application latency
#define _KEY 0x1A               //   Sets the pass key
#define _OPEN_ADV_PIPE 0x1B     //   Sets the pipes that are used for advertising data
#define _BROADCAST 0x1C         //   Enables a pipe to start sending advertisment data on non-connectable advertisment packets
#define _BOND_SEC_REQ 0x1D      //   Sends the security Manager Protocol (SMP)
#define _DIR_CONNECT 0x1E       //   Initiates directed advertisment
#define _CLOSE_REM_PIPE 0x1F    //   Closes a remote receive pipe from a peer device


/*DATA COMMANDS*/

#define _SET_LOCAL_DATA 0x0D    //   Sets local uint8_tacteristic value or uint8_tacteristic descriptor
#define _SEND_DATA 0x15         //   Sends data to a peer device through a transmit service pipe
#define _SEND_DATA_ACK 0x16     //   Comfirms reception of data from a peer device
#define _REQ_DATA 0x17          //   Requests data from a peer device through a service receive pipe
#define _SEND_DATA_NACK 0x18    //   Negaively acknowledges (nack) reception of data from a peer device


/*SYSTEM EVENTS*/

#define _DEV_START_EV 0x81      //   Indicates rest recovery or a state change
#define _ECHO_EV 0x82           //   Returns an identical copy of the PDU sent using the ECHO command in test mode
#define _HW_ERROR_EV 0x83       //   Returns hardware error debug information
#define _COMM_RESP_EV 0x84      //   Confirms reception or execution of an ACI command
#define _CONNECTED_EV 0x85      //   Indicates that a connection has been established with a peer device
#define _DISCONNECTED_EV 0x86   //   Indicates the loss of a connection
#define _BOND_STAT_EV 0x87      //   Returns the bonding procedure execution status
#define _PIPE_STAT_EV 0x88      //   Lists the pipe connection and availibility status
#define _TIM_EV 0x89            //   Returns the current connection timing information upon change of parameters
#define _DISP_KEY_EV 0x8E       //   Requests the application controller to display the 6 digit pass key
#define _KEY_REQ_EV 0x8F        //   Requests the controller to enter the pass key


/*DATA EVENTS*/

#define _DATA_CRED_EV 0x8A      //   Returns data command buffer credits
#define _DATA_ACK_EV 0x8B       //   Indicates reception of data by the peer device
#define _DATA_REC_EV 0x8C       //   Indicates that data has been received from the peer device
#define _PIPE_ERROR_EV 0x8D     //   Reports a pipe transmission failture

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

extern int8_t readBuff[32];
extern int8_t open_adv_pipe[10];
extern int8_t bond[6];
extern int8_t connect[6];
extern int8_t Directconnect[2];
extern int8_t broadcast[6];
extern int8_t set_local_data[4];
extern uint8_t  sendData[];

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void InitGPIOInt();
void Init_MCU();
void BlueLE_Write(uint8_t *dat, uint16_t len);
void BlueLE_Read();
void BlueLE_Setup();
uint8_t BlueLE_WaitForEvent(uint8_t event);
//extern void UART_Write_Line(uint8_t *line);
extern void BlueLE_Bond(uint16_t timeout, uint16_t adv_interval);
extern void BlueLE_SendData(uint8_t *dat, uint8_t len, uint8_t pipe);
extern void BlueLe_DirectConnect();
extern void BlueLe_Connect();
extern void BlueLE_TestModeState(char state);
extern void BlueLE_SendDataAck(uint8_t pipe);
extern void BlueLE_OpenRemotePipe(uint8_t pipe);
extern void BlueLE_SetLocalData(char *dat, char len, char pipe);
extern void BlueLE_GetTemp();
extern char BlueLE_DecodeTemp();

//rega
extern void BlueLe_Disconnect();
extern char BlueLE_EventCode();

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct
{
  uint8_t status_byte;
  uint8_t buffer[32];
} TSetupMsg;